/*******************************************************************************
 *  PROJECT: GNOME Colorscheme
 *
 *  AUTHOR: Jonathon Jongsma
 *
 *  Copyright (c) 2005 Jonathon Jongsma
 *
 *  License:
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the 
 *    Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 *    Boston, MA  02111-1307  USA
 *
 *******************************************************************************/

#include "gcs-scheme.h"

namespace gcs
{

    Scheme::Scheme(Color c, tSchemeType t)
        : m_pColor(new Color(c)), m_schemeType(t)
    {
        init();
    }


    void Scheme::init()
    {
        // clear the list of colors first.
        clear();
        switch (m_schemeType)
        {
            // for a description of the scheme types, see
            // http://www.color-wheel-pro.com/color-schemes.html
            case SCHEME_COMPLEMENTS:    // only two colors
                {
                    Color complement(*m_pColor);    // make a copy
                    complement.set_hue(m_pColor->get_hue() + (maxHueValue / 2));

                    push_back(*m_pColor);
                    push_back(complement);
                }
                break;

            case SCHEME_SPLIT_COMPLEMENTS:
                // three colors
                {
                    // angle to offset from 180 degrees
                    // tweak this number to get it right
                    const int offset = maxHueValue / 15;
                    Color scomplement1(*m_pColor), scomplement2(*m_pColor);
                    scomplement1.set_hue(m_pColor->get_hue() + (maxHueValue / 2) - offset);
                    scomplement2.set_hue(m_pColor->get_hue() + (maxHueValue / 2) + offset);

                    push_back(*m_pColor);
                    push_back(scomplement1);
                    push_back(scomplement2);
                }
                break;

            case SCHEME_TETRADS:
                // four colors (two complementary colors)
                {
                    // tweak this number to get it right
                    const int offset = maxHueValue / 6;
                    Color tetrad1(*m_pColor), tetrad2(*m_pColor), tetrad3(*m_pColor);
                    tetrad1.set_hue(m_pColor->get_hue() + offset);
                    tetrad2.set_hue(m_pColor->get_hue() + maxHueValue / 2);
                    tetrad3.set_hue(m_pColor->get_hue() + maxHueValue / 2 + offset);

                    push_back(*m_pColor);
                    push_back(tetrad1);
                    push_back(tetrad2);
                    push_back(tetrad3);
                }
                break;

            case SCHEME_ANALOGOUS:
                // three colors next to eachother on a color wheel
                {
                    // tweak this number to get it right
                    const int offset = maxHueValue / 12;
                    Color analogous1(*m_pColor), analogous2(*m_pColor);
                    analogous1.set_hue(m_pColor->get_hue() + offset);
                    analogous2.set_hue(m_pColor->get_hue() - offset);

                    push_back(*m_pColor);
                    push_back(analogous1);
                    push_back(analogous2);
                }
                break;

            case SCHEME_MONOCHROMATIC:
                // several colors of a similar hue
                {
                    Color mono1(*m_pColor), mono2(*m_pColor);

                    // this algorithm needs a bit of work
                    if (m_pColor->get_saturation() < maxSvValue / 2)
                    {
                        mono1.set_saturation(
                                (m_pColor->get_saturation() + maxSvValue) / 2);
                    }
                    else
                    {
                        mono1.set_saturation(m_pColor->get_saturation() / 2);
                    }

                    if (m_pColor->get_value() < maxSvValue / 2)
                    {
                        mono2.set_value(
                                (m_pColor->get_value() + maxSvValue) / 2);
                    }
                    else
                    {
                        mono2.set_value(m_pColor->get_value() / 2);
                    }

                    push_back(*m_pColor);
                    push_back(mono1);
                    push_back(mono2);
                }
                break;

            case SCHEME_TRIADS:
            default:
                // three colors
                {
                    const int offset = maxHueValue / 3;
                    Color triad1(*m_pColor), triad2(*m_pColor);
                    triad1.set_hue(m_pColor->get_hue() + offset);
                    triad2.set_hue(m_pColor->get_hue() - offset);

                    push_back(*m_pColor);
                    push_back(triad1);
                    push_back(triad2);
                }
                break;
        }
    }

    void Scheme::set_color(Color c)
    {
        m_pColor.reset(new Color(c));
        init();
    }


    void Scheme::set_scheme_type(tSchemeType t)
    {
        m_schemeType = t;
        init();
    }


    bool Scheme::operator==(const Scheme& other)
    {
        return (*m_pColor == *(other.m_pColor)
                && m_schemeType == other.m_schemeType);
    }

} // namespace gcs
