/*******************************************************************************
 *  PROJECT: GNOME Colorscheme
 *
 *  AUTHOR: Jonathon Jongsma
 *
 *  Copyright (c) 2005 Jonathon Jongsma
 *
 *  License:
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the 
 *    Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 *    Boston, MA  02111-1307  USA
 *
 *******************************************************************************/

#include "gcs-bookmarklist.h"
#include "gcs-cellrendererswatch.h"
#include "../gcs-debug.h"

namespace gcs
{
    namespace Widgets
    {
        BookmarkList::BookmarkList(void)
            : m_refListStore(Gtk::ListStore::create(m_columns))
        {
            set_model(m_refListStore);

            // display column headers
            set_headers_visible(true);

            // Create a instance of a custom CellRenderer for drawing color
            // swatches
            CellRendererSwatch *const pRenderer = Gtk::manage(new
                    CellRendererSwatch);
            // Create a new Column named "Favorites"
            Gtk::TreeViewColumn *const pColumn = Gtk::manage(new
                    Gtk::TreeViewColumn("Favorites"));
            // Pack the custom swatch renderer into the column
            pColumn->pack_start(*pRenderer, false);
            // associate the 'color' property of the custom renderer with the
            // value of m_columns.m_colText so that the cell renderer can color
            // each swatch according to the bookmarked item
            pColumn->add_attribute(pRenderer->property_color(),
                    m_columns.m_colText);
            // add a the model text column to the TreeView column -- This packs
            // both the swatch and the hexstring into the same treeview column
            pColumn->pack_start(m_columns.m_colText);
            // append the column to the TreeView
            append_column(*pColumn);
        }


        void BookmarkList::add(Color clr)
        {
            // Assume the the color we want to add is not already in the list
            bool exists = false;
            // Loop through all the items in the list 
            for (Gtk::TreeModel::iterator iter = m_refListStore->children().begin();
                    iter != m_refListStore->children().end();
                    ++iter)
            {
                // check if the current item in the list matches the added
                // color value
                if ((*iter)[m_columns.m_colText] == clr.get_hexstring())
                {
                    exists = true;
                    get_selection()->select(iter);
                    break;
                }
            }
            if (!exists)
            {
                // add a new row to the end
                Gtk::TreeModel::iterator iter = m_refListStore->append();
                Gtk::TreeModel::Row row = *iter;
                row[m_columns.m_colText] = clr.get_hexstring();

                //At one point I wanted to select the rows as soon as they were
                //added since that's the one we're currently set to, but that
                //caused some infinite looping, so I've disabled it for now
                //until I figure out a better way
                //
                Glib::RefPtr<Gtk::TreeView::Selection> sel = get_selection();
                sel->select(iter);
            }
        }

        // returns the hexstring of the currently selected row
        tHexString BookmarkList::get_hexstring(void)
        {
            tHexString hex;
            Glib::RefPtr<Gtk::TreeSelection> refSel = get_selection();
            Gtk::TreeModel::iterator iter = refSel->get_selected();
            if (iter)
            {
                Gtk::TreeModel::Row row = *iter;
                hex =  row[m_columns.m_colText];
            }
            return hex;
        }


        // clear out the bookmarks list
        void BookmarkList::clear(void)
        {
            get_selection()->unselect_all();
            m_refListStore->clear();
        }


        // check if the list is empty
        bool BookmarkList::empty(void)
        {
            return m_refListStore->children().empty();
        }


        void BookmarkList::remove_selected(void)
        {
            Glib::RefPtr<Gtk::TreeSelection> sel = get_selection();
            Gtk::TreeModel::iterator iter = sel->get_selected();
            sel->unselect(iter);
            m_refListStore->erase(iter);
        }


        gint BookmarkList::count_selected(void)
        {
            return get_selection()->count_selected_rows();
        }

    }   // namespace Widgets 

}   // namespace gcs
